package mi.hdm.recipes;

import mi.hdm.exceptions.InvalidRecipeException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class RecipeManager {
    //TODO observable array list

    private final List<Recipe> recipes;
    private final CategoryManager categories;
    private final IngredientManager ingredients;
    private static final Logger log = LogManager.getLogger(RecipeManager.class);

    public RecipeManager(List<Recipe> recipes, CategoryManager categories, IngredientManager ingredients) {
        this.recipes = recipes;
        this.categories = categories;
        this.ingredients = ingredients;
    }

    public RecipeManager() {
        recipes = new ArrayList<>();
        categories = CategoryManager.getInstance();
        ingredients = IngredientManager.getInstance();
    }

    public void addRecipe(Recipe recipe) {
        if (getRecipeByName(recipe.getName()).isEmpty()) {
            recipes.add(recipe);
            log.info("Recipe {} added successfully.", recipe.getName());
        } else {
            log.error("Recipe {} not added because it already exists.", recipe.getName());
            throw new InvalidRecipeException("Recipe name already exists");
        }
    }

    public Recipe deleteRecipe(int i) {
        log.info("Recipe {} deleted successfully.", recipes.get(i).getName());
        return recipes.remove(i);
    }

    public void deleteRecipe(String name) {
        Recipe r = getRecipeByName(name).orElseThrow(() -> new InvalidRecipeException("A recipe with this name does not exist"));
        log.info("Recipe {} deleted successfully.", name);
        recipes.remove(r);
    }

    // Exception vs return false: What's the best approach?
    // -> false for normal situations, exceptions for real faults which should not occur
    public void deleteRecipe(Recipe r) {
        log.info("Recipe deleted successfully.");
        if (recipes.remove(r) == false) {
            throw new InvalidRecipeException("Recipe is not listed.");
        }
    }

    private Optional<Recipe> getRecipeByName(String name) {
        for (final Recipe r : recipes) {
            if (name.equals(r.getName())) {
                return Optional.of(r);
            }
        }
        return Optional.empty();
    }

    public List<Recipe> getRecipes() {
        return recipes;
    }
}
