package mi.hdm.shoppingList;

import mi.hdm.exceptions.InvalidIngredientException;
import mi.hdm.recipes.Ingredient;
import mi.hdm.recipes.Recipe;
import mi.hdm.recipes.ValidObjectsPool;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

class ShoppingListTest {
    private final ShoppingList underTest = ShoppingList.getInstance();
    private final static Ingredient ing1 = ValidObjectsPool.getValidIngredientOne();
    private final static Ingredient ing2 = ValidObjectsPool.getValidIngredientTwo();

    private final static Recipe r1 = ValidObjectsPool.getValidRecipeOne();

    @BeforeEach
    public void setup() {
        underTest.clear();
    }

    @Test
    public void shouldAddIngredient() {
        //given
        Map<Ingredient, Boolean> expected = Map.of(ing1, false);

        //when
        underTest.addToShoppingList(ing1);

        //expect
        assertEquals(expected, underTest.getList());
    }

    @Test
    public void shouldAddAllIngredients() {
        //given
        Map<Ingredient, Boolean> expected = Map.of(ing1, false, ing2, false);

        //when
        underTest.addAllToShoppingList(r1);

        //expect
        assertEquals(expected, underTest.getList());
    }

    @Test
    public void testSetStatusByObject() {
        //given
        Map<Ingredient, Boolean> expected = Map.of(ing1, true);
        underTest.addToShoppingList(ing1);

        //when
        underTest.flipStatus(ing1);

        //expect
        assertEquals(expected, underTest.getList());
        assertThrows(InvalidIngredientException.class, () -> underTest.flipStatus(ing2));
        assertThrows(NullPointerException.class, () -> underTest.flipStatus((Ingredient) null));
    }

    @Test
    public void testSetStatusByName() {
        //given
        Map<Ingredient, Boolean> expected = Map.of(ing1, true);
        underTest.addToShoppingList(ing1);

        //when
        underTest.flipStatus(ing1.getName());

        //expect
        assertEquals(expected, underTest.getList());
        assertThrows(InvalidIngredientException.class, () -> underTest.flipStatus(ing2.getName()));
        assertThrows(NullPointerException.class, () -> underTest.flipStatus((String) null));
    }

    @Test
    public void shouldRemoveAllBoughtItems() {
        //given
        Map<Ingredient, Boolean> expected = Map.of();
        underTest.addToShoppingList(ing1);
        underTest.flipStatus(ing1);

        //when
        underTest.removeAllBought();

        //expect
        assertEquals(expected, underTest.getList());
    }
}